-- Part of SmallEiffel -- Read DISCLAIMER file -- Copyright (C) 
-- Dominique COLNET and Suzanne COLLIN -- colnet@loria.fr
--
class STD_FILE_WRITE
--
-- Basic output facilities to write a named file on the disk.
--
-- Note : most features are common with STD_OUTPUT so you can 
--        test your program first on the screen and then, changing 
--        of instance (STD_OUTPUT/STD_FILE_WRITE), doing the same
--        on a file.
--
   
inherit OUTPUT_STREAM
   
creation 
   connect_to, make

feature
   
   path: STRING;
	 -- Not Void when connected to the corresponding file
	 -- on the disk.

feature {NONE}
   
   output_stream: POINTER;

feature 

   connect_to(new_path: STRING) is
      require
	 not is_connected;
	 not new_path.empty
      local
	 p: POINTER;
      do
         make;
	 p := new_path.to_external;
	 output_stream := sfw_open(new_path.count,p);
	 if output_stream.is_not_void then
	    path := new_path;
	 end;
      end;
   
feature

   disconnect is
      require
	 is_connected
      do
	 fclose(output_stream); 
	 path := Void;
      end;
   
   make is
      do
      end;

feature

   is_connected: BOOLEAN is
      do
	 Result := path /= Void;
      end;

   flush is
      do
	 flush_stream(output_stream);
      end;

   put_character(c: CHARACTER) is
      do
	 write_byte(output_stream,c);
      end;

feature {NONE}
   
   sfw_open(path_count: INTEGER; path_pointer: POINTER): POINTER is
      external "SmallEiffel"
      end;

   fclose(stream_pointer : POINTER) is
      external "C_InlineWithoutCurrent"
      end;

end
